require 'spec_helper'

describe Metamorphosis::PrefixParser do
  let!(:available_units) {Metamorphosis::SI_UNITS }
  let!(:prefixes) { Metamorphosis::DEFAULT_PREFIXES }
  let!(:parser) {Metamorphosis::PrefixParser.new(prefixes, available_units)}

  [:meter, :joule].each do |unit|
    Metamorphosis::DEFAULT_PREFIXES.each do |prefix, value|
      new_unit = "#{prefix}#{unit}"
      it "parses #{new_unit}as [#{value}, #{unit}]" do
        parser.parse(new_unit).should == [value, unit.to_sym]
      end
    end
  end

  it "should parse kJ" do
    parser.parse("kJ").should == [1000.0, :J]
  end

  it "should return the unit with value of 1.0 for unknown prefix" do
    parser.parse("crazystuff").should == [1.0, :crazystuff]
  end

  it "should return the unit with value of 1.0 for unknown unit" do
    parser.parse("kilojunk").should == [1.0, :kilojunk]
  end

  it "should allow prefixes to be overridden" do
    prefixes = { :yotta => 2**80, :Y => 2**80,
                 :zetta => 2**70, :Z => 2**70,
                 :exa   => 2**60, :E => 2**60,
                 :peta  => 2**50, :P => 2**50,
                 :tera  => 2**40, :T => 2**40,
                 :giga  => 2**30, :G => 2**30,
                 :mega  => 2**20, :M => 2**20,
                 :kilo  => 2**10, :k => 2**10 }
    parser = Metamorphosis::PrefixParser.new(prefixes, [:byte])
    parser.parse("kbyte").should == [1024, :byte]

    parser.parse("Mbyte").should == [1048576, :byte]
  end
end
