require 'spec_helper'

describe Metamorphosis::UnitRegistry do

  let!(:registry) { Metamorphosis::UnitRegistry.new }
  let!(:subject)  { registry }

  before do
    registry.register(:wegowise, :btu, 1)
    registry.register(:wegowise, :therms, 100_000)
    registry.register(:other, :btu, 22)
    registry.register(:junk, :whatchamakalit, 0.0000001)
  end

  its(:category_names) { should match_array [:wegowise, :other, :junk] }

  it "keeps different conversion factors for units across categories" do
    registry.conversion_factor(:wegowise, :btu).
      should_not == registry.conversion_factor(:other, :btu)
  end

  it "returns the set of categories a unit belongs to" do
    registry.unit_categories(:btu).to_a.should match_array [:wegowise, :other]
    registry.unit_categories(:therms).to_a.should == [:wegowise]
  end

  it "tracks conversion factors for units" do
    registry.conversion_factor(:wegowise, :btu).should == 1
    registry.conversion_factor(:wegowise, :therms).should == 100_000
    registry.conversion_factor(:other, :btu).should == 22
  end

  it "tracks which units have been registered" do
    registry.include?(:btu).should be_true
    registry.include?(:therms).should be_true
    registry.include?(:joules).should be_false
  end
end
