require 'spec_helper'

describe Metamorphosis::UnitValue do

  before{ Metamorphosis.register_predefined_units }
  after { Metamorphosis.reset! }

  let!(:unit) { Metamorphosis::UnitValue.new('therms', 1.3, 1_000) }
  subject{ unit }

  its(:unit_symbol){ should == :therms }
  its(:exponent){ should == 1_000 }
  its(:value){ should == 1.3 }
  its(:to_s){ should == '1300.0 therms' }
  its(:to_f){ should == 1300.0 }

  # Comparisons between the same unit
  it{ should be == Metamorphosis::UnitValue.new(:therms, 1_300)  }
  it{ should be >  Metamorphosis::UnitValue.new(:therms, 1_290) }
  it{ should be <  Metamorphosis::UnitValue.new(:therms, 1_310)  }

  # Comparisons between different units
  it{ should be >  Metamorphosis::UnitValue.new(:J, 1_000)  }
  it{ should be ==  Metamorphosis::UnitValue.new(:J, 1.05506e+8 * 1_300)  }
  it{ should be <  Metamorphosis::UnitValue.new(:J, 1.05506e+8 * 1_400)  }

  # Should not compare units that are not alike (i.e. energy vs. distance)
  it{ should_not eq(Metamorphosis::UnitValue.new(:meters, 100))  }

  it "compares like units" do
    Metamorphosis::UnitValue.new('therms', 85.30320550490019).should ==
      Metamorphosis::UnitValue.new('kilowatt_hour', 2_500)
  end

  # Don't compare non unit like things
  it{ should_not be ==  :x }

  it "raises ArgumentError comparing non numeric/UnitValues with UnitValue" do
    expect{ unit <=>  :x }.to raise_error(ArgumentError)
  end

  # Assume numeric values are of the same unit
  it { should be == 1_300 }


  it "returns itself when to/as method receives no parameters" do
    unit.to.should === unit
    unit.as.should === unit
  end

  it "delegates conversion to the Metamorphosis::Engine" do
    Metamorphosis.engine.should_receive(:convert_to).with(:J, unit)
    unit.to(:J)
  end

  it "delegates missing methods to the Metamorphosis::Engine" do
    Metamorphosis.engine.should_receive(:convert_to).with(:J, unit).
      and_return("cooking with fire!")
    unit.to(:J).should == "cooking with fire!"
  end

  it "is a good method_missing citizen when Metamorphosis doesn't know how
      to convert to the given unit" do
    Metamorphosis.engine.should_receive(:convert_to).with(:J, unit).
      and_return(nil)
    expect{ unit.J }.to raise_error(NoMethodError)
  end

end
